
import { getSearchRegs, joinMainSearchkey } from '@/utils/utils-custom.js'

export default class SettingUtils {
  /**
   * @description 生成页面查询语句
   * @return {String}  scripts       自定义脚本
   * @return {String}  searches      搜索条件
   * @return {Object}  setting       页面设置
   * @return {Array}   columns       显示字段
   */
  static getDebugSql (setting, scripts, columns, searches = [], type) {
    let sql = ''
    let error = ''
    let _dataresource = ''
    let arr_field = columns.map(item => item.field).join(',')
    let _customScript = ''
    let _tailScript = ''
    scripts && scripts.forEach(script => {
      if (script.status === 'false') return
      if (script.position !== 'back') {
        _customScript += `
        ${script.sql}
        `
      } else {
        _tailScript += `
        ${script.sql}
        `
      }
    })

    if (!arr_field) {
      arr_field = '*'
    }

    if (setting.interType === 'system' && setting.execute !== 'false') {
      _dataresource = setting.dataresource || ''
    }

    if (/\s/.test(_dataresource)) {
      _dataresource = '(' + _dataresource + ') tb'
    }
    
    if (window.GLOB.funcs && window.GLOB.funcs.length > 0) {
      window.GLOB.funcs.forEach(item => {
        let reg = new RegExp('\\$ex@' + item.func_code + '@ex\\$', 'ig')
        _dataresource = _dataresource.replace(reg, `/*$ex@${item.func_code}-begin*/\n${item.key_sql}\n/*@ex$-end*/`)
        _customScript = _customScript.replace(reg, `/*$ex@${item.func_code}-begin*/\n${item.key_sql}\n/*@ex$-end*/`)
      })
    }

    _dataresource = _dataresource.replace(/@(BID|LoginUID|SessionUid|UserID|Appkey|time_id|datam|upid|typename)@/ig, `'1949-10-01 15:00:00'`)
    _customScript = _customScript.replace(/@(BID|LoginUID|SessionUid|UserID|Appkey|time_id|datam|upid|typename)@/ig, `'1949-10-01 15:00:00'`)
    _tailScript = _tailScript.replace(/@(BID|LoginUID|SessionUid|UserID|Appkey|time_id|datam|upid|typename)@/ig, `'1949-10-01 15:00:00'`)

    _dataresource = _dataresource.replace(/@\$|\$@/ig, '')
    _customScript = _customScript.replace(/@\$|\$@/ig, '')
    _tailScript = _tailScript.replace(/@\$|\$@/ig, '')

    if (window.GLOB.process) {
      _dataresource = _dataresource.replace(/@works_flow_code@/ig, `'1949-10-01 15:00:00'`)
      _customScript = _customScript.replace(/@works_flow_code@/ig, `'1949-10-01 15:00:00'`)
      _tailScript = _tailScript.replace(/@works_flow_code@/ig, `'1949-10-01 15:00:00'`)
    }

    _dataresource = _dataresource.replace(/@select\$|\$select@/ig, '')
    _customScript = _customScript.replace(/@select\$|\$select@/ig, '')
    _customScript = _customScript.replace(/@sum\$|\$sum@/ig, '')
    _dataresource = _dataresource.replace(/\$sum@/ig, '/*')
    _dataresource = _dataresource.replace(/@sum\$/ig, '*/')

    if (type === 'calendar') {
      _dataresource = _dataresource.replace(/@mk_year@/ig, '')
      _customScript = _customScript.replace(/@mk_year@/ig, '')
    }

    if (_customScript) {
      _customScript = `declare @ErrorCode nvarchar(50),@retmsg nvarchar(4000),@UserName nvarchar(50),@FullName nvarchar(50),@RoleID nvarchar(512),@mk_departmentcode nvarchar(512),@mk_organization nvarchar(512),@mk_user_type nvarchar(20),@mk_nation nvarchar(50),@mk_province nvarchar(50),@mk_city nvarchar(50),@mk_district nvarchar(50),@mk_address nvarchar(100)
        select @ErrorCode='',@retmsg ='',@UserName='', @FullName='', @RoleID='', @mk_departmentcode='', @mk_organization='', @mk_user_type='', @mk_nation='', @mk_province='', @mk_city='', @mk_district='', @mk_address=''
        ${_customScript}
      `
    }

    // 外联数据库替换
    if (window.GLOB.externalDatabase !== null) {
      _dataresource = _dataresource.replace(/@db@/ig, window.GLOB.externalDatabase)
      _customScript = _customScript.replace(/@db@/ig, window.GLOB.externalDatabase)
      _tailScript = _tailScript.replace(/@db@/ig, window.GLOB.externalDatabase)
    }

    if (window.GLOB.urlFields) {
      window.GLOB.urlFields.forEach(field => {
        let reg = new RegExp('@' + field + '@', 'ig')
        _dataresource = _dataresource.replace(reg, `'0'`)
        _customScript = _customScript.replace(reg, `'0'`)
        _tailScript = _tailScript.replace(reg, `'0'`)
      })
    }

    // 正则替换
    let custompage = /@pageSize@|@orderBy@/i.test(_dataresource + _customScript)
    let _regoptions = getSearchRegs(searches)

    let _search = joinMainSearchkey(searches)

    _regoptions.push({
      reg: new RegExp('@userName@', 'ig'),
      value: `'mk'`
    }, {
      reg: new RegExp('@fullName@', 'ig'),
      value: `'mk'`
    }, {
      reg: new RegExp('@orderBy@', 'ig'),
      value: setting.order || ''
    }, {
      reg: new RegExp('@pageSize@', 'ig'),
      value: 9999
    }, {
      reg: new RegExp('@pageIndex@', 'ig'),
      value: 1
    })

    _regoptions.forEach(item => {
      _dataresource = _dataresource.replace(item.reg, item.value || '0')
      _customScript = _customScript.replace(item.reg, item.value || '0')
      _tailScript = _tailScript.replace(item.reg, item.value || '0')
    })

    if (setting.queryType === 'statistics') {
      _search = ''
    }

    // 数据源处理, 存在显示列时 
    if (_dataresource) {
      if (custompage) {
        _dataresource = `/*system_query*/select ${arr_field} from ${_dataresource} ${_search}`
      } else if (setting.laypage === 'true' && setting.order) {
        _dataresource = `/*system_query*/select top 10 ${arr_field} from (select ${arr_field} ,ROW_NUMBER() over(order by ${setting.order}) as rows from ${_dataresource} ${_search}) tmptable where rows > 0 order by tmptable.rows`
      } else if (setting.order) {
        _dataresource = `/*system_query*/select ${arr_field} from (select ${arr_field} ,ROW_NUMBER() over(order by ${setting.order}) as rows from ${_dataresource} ${_search}) tmptable order by tmptable.rows`
      } else {
        _dataresource = `/*system_query*/select ${arr_field} from ${_dataresource} ${_search}`
      }
    }

    if (_customScript) {
      sql = `${_customScript}
        ${_dataresource}
        ${_tailScript}
        aaa:
        if @ErrorCode!=''
          insert into tmp_err_retmsg (ID, ErrorCode, retmsg, CreateUserID) select @time_id@,@ErrorCode, @retmsg,'1949-10-01 15:00:00'
      `
    } else if (_tailScript) {
      sql = `declare @ErrorCode nvarchar(50),@retmsg nvarchar(4000),@UserName nvarchar(50),@FullName nvarchar(50),@RoleID nvarchar(512),@mk_departmentcode nvarchar(512),@mk_organization nvarchar(512),@mk_user_type nvarchar(20),@mk_nation nvarchar(50),@mk_province nvarchar(50),@mk_city nvarchar(50),@mk_district nvarchar(50),@mk_address nvarchar(100)
        select @ErrorCode='',@retmsg ='',@UserName='', @FullName='', @RoleID='', @mk_departmentcode='', @mk_organization='', @mk_user_type='', @mk_nation='', @mk_province='', @mk_city='', @mk_district='', @mk_address=''
        ${_dataresource}
        ${_tailScript}
        aaa:
        if @ErrorCode!=''
          insert into tmp_err_retmsg (ID, ErrorCode, retmsg, CreateUserID) select @time_id@,@ErrorCode, @retmsg,'1949-10-01 15:00:00' 
      `
    } else {
      sql = `declare @ErrorCode nvarchar(50),@retmsg nvarchar(4000),@UserName nvarchar(50),@FullName nvarchar(50),@RoleID nvarchar(512),@mk_departmentcode nvarchar(512),@mk_organization nvarchar(512),@mk_user_type nvarchar(20),@mk_nation nvarchar(50),@mk_province nvarchar(50),@mk_city nvarchar(50),@mk_district nvarchar(50),@mk_address nvarchar(100)
        select @ErrorCode='',@retmsg ='',@UserName='', @FullName='', @RoleID='', @mk_departmentcode='', @mk_organization='', @mk_user_type='', @mk_nation='', @mk_province='', @mk_city='', @mk_district='', @mk_address=''
        ${_dataresource}`
    }

    let errors = []

    if (/@[0-9a-zA-Z_]+@/ig.test(sql)) {
      let arr = sql.match(/@[0-9a-zA-Z_]+@/ig)

      let getTrueSql = (sl) => {
        if (!/\/\*/.test(sl)) return sl
        let index = 0

        sl = sl.replace(/\s/ig, ' ')
        sl = sl.replace(/\*\//ig, 'm_k@')
        sl = sl.replace(/\/\*/ig, () => {
          index++
          return '@m_k' + index
        })

        for (let i = index; i > 0; i--) {
          let reg = new RegExp(`@m_k${i}.+m_k@`, 'ig')
          sl = sl.replace(reg, '')
        }

        return sl
      }

      let _scripts = []
      _dataresource = _dataresource.replace('/*system_query*/', '')
      _dataresource = getTrueSql(_dataresource)
      scripts.forEach(item => {
        _scripts.push({...item, sql: getTrueSql(item.sql)})
      })

      arr.forEach(item => {
        if (/@time_id@/ig.test(item)) return

        let reg = new RegExp(item, 'i')
        if (reg.test(_dataresource)) {
          errors.push(`数据源中存在未替换值${item}`)
        }
        _scripts.forEach(script => {
          if (reg.test(script.sql)) {
            if (script.$index) {
              errors.push(`自定义脚本(${script.$index})存在未替换值${item}`)
            } else {
              errors.push(`自定义脚本中存在未替换值${item}`)
            }
          }
        })
      })
    }

    return { error, sql, errors: errors.join('；'), custompage }
  }
}